import os
from ipywidgets import widgets, Layout, HBox

# widgets
xnatserver = widgets.Text(description="XNAT SERVER", tooltip='URL to XNAT', value='', width=200)
prjid = widgets.Text(
    description="PRJID",
    tooltip='Leave this empty when defining non-project dicom dir',
    value='PRJ1607_TEPT',
    width=200,
    layout=Layout(disabled='disabled')
)
dicomdir = widgets.Text(description="DICOM DIR", value='', width=200, layout=Layout(width='70%'))
subject    = widgets.Text(description="SUBJECT ID (prefix)", value='S', width=200)

# event handling
def on_change(b):
    dicomdir.value = ""

prjid.observe(on_change,names='value')
on_change(None)

def update_ss(b):
    p = dicomdir.value
    ss = [(x, os.path.join(p,x)) for x in sorted(os.listdir(p), reverse=True) if x.startswith(subject.value) and os.path.isdir(os.path.join(p,x))]
    selectedss.options = dict(ss)

subjbutton = widgets.Button(
    description='List subject matches',
    disabled=False,
    button_style='', # 'success', 'info', 'warning', 'danger' or ''
    tooltip='',
    icon='refresh',
    layout=Layout(width='30%'),
)
subjbutton.on_click(update_ss)

selectedss = widgets.SelectMultiple(
    options=[],
    value=[],
    #rows=10,
    description='Subjects to send',
    disabled=False,
    layout={'height': '200px'}
)
subject.observe(update_ss,names='value')
    
login    = widgets.Text(description="XNAT login", value='', width=200)
password = widgets.Password(description="PASSWORD", value='', width=200)

sendbutton = widgets.Button(
    description='Send to XNAT',
    disabled=False,
    button_style='warning', # 'success', 'info', 'warning', 'danger' or ''
    tooltip='Sends selected subjects to xnat',
    icon='check',
    layout=Layout(width='30%')
)

def on_send_clicked(b):
    ss_fp = [(k,v) for k,v in selectedss.options.items() if v in selectedss.value]
    print("Sending to XNAT")
    print(ss_fp)
    print(prjid.value)
    print(login.value)
    #send_to_xnat(ss_fp, xnatserver.value, prjid.value, login.value, password.value)
    #send_logs_to_xnat(ss_fp, xnatserver.value, prjid.value, login.value, password.value)
    
sendbutton.on_click(on_send_clicked)

# display
def display_form():
    display(xnatserver)
    display(prjid)
    display(dicomdir)
    display(subject)
    display(subjbutton)
    display(selectedss)
    display(HBox([login, password]))
    display(sendbutton)


import os, shutil, tempfile, bz2

# Function to manipulate bz files
def unbzip( filepath, dest=None ):
    newfile = filepath.rsplit('.', 1)[0]
    # If the results will be moved to other destination
    if dest:
        newfile = os.path.join(dest, os.path.basename(newfile))

    with open(newfile, 'wb') as new_file, bz2.BZ2File(filepath, 'rb') as file:
        for data in iter(lambda : file.read(100 * 1024), b''):
            new_file.write(data)

# Function to create a temporary zip file
# data_in = folder if is a string or files if a list
# Ignores subfolders
def tmp_zip( data_in, prefix='xnat_' ):
    tmpdir = tempfile.mkdtemp()
    try:
        files = os.listdir(data_in) if isinstance( data_in, str ) else data_in
        # Moving all files to temporary directory
        for subfile in files:
            # When data_in is a folder, it completes the path
            if isinstance( data_in, str ):
                subfile = os.path.join(data_in, subfile)
            # Uncompressing bz files
            # TODO: Uncompress gz and zip
            if subfile.endswith('.bz'):
                unbzip(subfile, tmpdir)
            else:
                shutil.copy2(subfile, tmpdir)

        # Creating temporary zip name
        fzip = tempfile.NamedTemporaryFile(prefix=prefix)
        fzip.close()

        shutil.make_archive(fzip.name, 'zip', tmpdir)
    finally:
        shutil.rmtree(tmpdir)
    
    return fzip.name + '.zip'


# XNATYPY: https://xnat.readthedocs.io/en/latest/
import xnat as xnatpy
import os, sys
from .util import tmp_zip

# Main interface with XNAT
# methods created to simplify XNAT access
class Xnat:
    def __init__(self, server, user, password):
        self.server   = server
        self.user     = user
        self.password = password
        self.__connect()

    # Uses interface from xnatpy
    def __connect(self):
        self.session = xnatpy.connect(self.server, user=self.user, password=self.password)

    # Listing all projects
    def list_projects(self):
        projects = []
        for project in self.session.projects.values():
            projects.append( (project.id, project.name) )
        return projects

    # Function to import resources
    def import_resource( self, obj, subdir, files ):
        for file in files:
            filename = os.path.basename(file)
            uri = '{}/resources/{}/files/{}?inbody=true'.format(obj.uri, subdir, filename)
            with open(file, 'rb') as f:
                self.session.put(uri, data=f.read())

    # function to send a specific sequence to xnat
    def send_sequence(self, project, subject, sequence_dir):
        zipfname = tmp_zip( sequence_dir )
        try:
            self.session.services.import_( zipfname,\
                overwrite='none',\
                project=project,\
                subject=subject,\
                trigger_pipelines=False )
        except:
            print("Unexpected error during XNAT import:")
            print(sys.exc_info())
        os.remove( zipfname )

    # function to send a complete session to xnat
    def send_session(self, project, subject, session_dir):
        sequences = os.listdir(session_dir)
        for (n, sequence) in enumerate(sequences):
            print("[{:02d}] Sending: {}".format(n, sequence))
            sequence_dir = os.path.join(session_dir, sequence)
            self.send_sequence( project, subject, sequence_dir )
            
        print('Finished!')


import os

import ipywidgets as widgets

class FileBrowser(object):
    def __init__(self):
        self.path = os.getcwd()
        self._update_files()
        
    def _update_files(self):
        self.files = list()
        self.dirs = list()
        if(os.path.isdir(self.path)):
            for f in os.listdir(self.path):
                ff = self.path + "/" + f
                if os.path.isdir(ff):
                    self.dirs.append(f)
                else:
                    self.files.append(f)
        
    def widget(self):
        box = widgets.VBox()
        self._update(box)
        return box
    
    def _update(self, box):
        
        def on_click(b):
            if b.description == '..':
                self.path = os.path.split(self.path)[0]
            else:
                self.path = self.path + "/" + b.description
            self._update_files()
            self._update(box)
        
        buttons = []
        if self.files:
            button = widgets.Button(description='..', background_color='#d0d0ff')
            button.on_click(on_click)
            buttons.append(button)
        for f in self.dirs:
            button = widgets.Button(description=f, background_color='#d0d0ff')
            button.on_click(on_click)
            buttons.append(button)
        for f in self.files:
            button = widgets.Button(description=f)
            button.on_click(on_click)
            buttons.append(button)
        box.children = tuple([widgets.HTML("<h2>%s</h2>" % (self.path,))] + buttons)

# example usage:
#   f = FileBrowser()
#   f.widget()
#   <interact with widget, select a path>
# in a separate cell:
#   f.path # returns the selected path



__all__ = ['neuroquant']

# Basic modules
import os, glob, re, warnings, shutil

# Checking if openpyxl exists
import importlib
if not importlib.util.find_spec("openpyxl"):
    os.system("pip install openpyxl")

if not importlib.util.find_spec("pydicom"):
    os.system("pip install -U pydicom")

# Module to manipule XLS files
from openpyxl import Workbook, load_workbook
# Module to read DICOM
import pydicom

class neuroquant:
    repo = ''
    xls  = ''
    
    def __init__( self, subj_data ):
        (self.subj, self.subjdir) = subj_data;
        self.__extract_subjid();
        self.__extract_dcm_header();
        
    def __extract_subjid( self ):
        self.subjid = self.subj.split('_')[0]
        
    def __extract_dcm_header( self ):
        items = glob.glob( '%s/*dcm' % self.get_3dt1_path() )
        dcm = pydicom.dcmread( items[0] )
        dir(dcm)
        self.PatientName = dcm.PatientName
        self.PatientBirthDate = dcm.PatientBirthDate
        self.PatientID = dcm.PatientID
        self.SeriesDate = dcm.SeriesDate
        
    def print_patient(self):
        print('Patient :     %s' % self.PatientName)
        print('ID Pac. :     %s' % self.PatientID)
        print('Birth Date :  %s' % self.PatientBirthDate)
        print('Series Date : %s' % self.SeriesDate)
        
    def nq_subjdir(self):
        return '%s/%s' % (Neuroquant.repo , self.subj)
    
    def mkdir( directory ):
        if not os.path.exists( directory ):
            os.mkdir( directory )

    # Copy 3DT1 to NeuroQuant repository
    def cp_3dt1(self):
        print("%s - Copying 3DT1" % self.subjid)
        folder_t1 = self.get_3dt1_path()
        out_dir = '%s/3DT1_orig' % self.nq_subjdir
        # Copy only if found 3DT1 image and destination directory don't exist
        if folder_t1 and not os.path.exists( out_dir ):
            Neuroquant.mkdir( self.nq_subjdir() )
            shutil.copytree( folder_t1, out_dir )
        
    def get_3dt1_path(self):
        items = glob.glob( '%s/t1_mprage*' % self.subjdir )
        if items:
            return items[0]
        return None
        
    def history(self):
        warnings.filterwarnings("ignore") # To surpress openpyxl warnings for xlsx files
        print('TODO: history')
        warnings.filterwarnings("default")


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="neuro_tools",
    version="0.1.1",
    author="Bruno Melo",
    author_email="bruno.melo@idor.org",
    description="Neuroimaging tools to help with BIDS, XNAT, workflows, etc.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/InstitutoDOr/neuro_tools",
    packages=setuptools.find_packages(),
    install_requires=[
          'pydicom',
          'openpyxl',
          'xnat',
      ],
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)